@@ -1,15 +1,19 @@
+# Authors: 
+#   Trevor Perrin
+#   Google - handling CertificateRequest.certificate_types
+#
+# See the LICENSE file for legal information regarding use of this file.
+
 """Classes representing TLS messages."""
 
-from utils.jython_compat import *
-from utils.cryptomath import *
-from errors import *
-from utils.codec import *
-from constants import *
-from X509 import X509
-from X509CertChain import X509CertChain
-
-import sha
-import md5
+from .utils.compat import *
+from .utils.cryptomath import *
+from .errors import *
+from .utils.codec import *
+from .constants import *
+from .x509 import X509
+from .x509certchain import X509CertChain
+from .utils.tackwrapper import *
 
 class RecordHeader3:
     def __init__(self):
@@ -25,7 +29,7 @@ class RecordHeader3:
         return self
 
     def write(self):
-        w = Writer(5)
+        w = Writer()
         w.add(self.type, 1)
         w.add(self.version[0], 1)
         w.add(self.version[1], 1)
@@ -56,22 +60,7 @@ class RecordHeader2:
         return self
 
 
-class Msg:
-    def preWrite(self, trial):
-        if trial:
-            w = Writer()
-        else:
-            length = self.write(True)
-            w = Writer(length)
-        return w
-
-    def postWrite(self, w, trial):
-        if trial:
-            return w.index
-        else:
-            return w.bytes
-
-class Alert(Msg):
+class Alert:
     def __init__(self):
         self.contentType = ContentType.alert
         self.level = 0
@@ -90,29 +79,26 @@ class Alert(Msg):
         return self
 
     def write(self):
-        w = Writer(2)
+        w = Writer()
         w.add(self.level, 1)
         w.add(self.description, 1)
         return w.bytes
 
 
-class HandshakeMsg(Msg):
-    def preWrite(self, handshakeType, trial):
-        if trial:
-            w = Writer()
-            w.add(handshakeType, 1)
-            w.add(0, 3)
-        else:
-            length = self.write(True)
-            w = Writer(length)
-            w.add(handshakeType, 1)
-            w.add(length-4, 3)
-        return w
-
+class HandshakeMsg:
+    def __init__(self, handshakeType):
+        self.contentType = ContentType.handshake
+        self.handshakeType = handshakeType
+    
+    def postWrite(self, w):
+        headerWriter = Writer()
+        headerWriter.add(self.handshakeType, 1)
+        headerWriter.add(len(w.bytes), 3)
+        return headerWriter.bytes + w.bytes
 
 class ClientHello(HandshakeMsg):
     def __init__(self, ssl2=False):
-        self.contentType = ContentType.handshake
+        HandshakeMsg.__init__(self, HandshakeType.client_hello)
         self.ssl2 = ssl2
         self.client_version = (0,0)
         self.random = createByteArrayZeros(32)
@@ -121,9 +107,11 @@ class ClientHello(HandshakeMsg):
         self.certificate_types = [CertificateType.x509]
         self.compression_methods = []   # a list of 8-bit values
         self.srp_username = None        # a string
+        self.tack = False
 
     def create(self, version, random, session_id, cipher_suites,
-               certificate_types=None, srp_username=None):
+               certificate_types=None, srp_username=None,
+               tack=False):
         self.client_version = version
         self.random = random
         self.session_id = session_id
@@ -131,6 +119,7 @@ class ClientHello(HandshakeMsg):
         self.certificate_types = certificate_types
         self.compression_methods = [0]
         self.srp_username = srp_username
+        self.tack = tack
         return self
 
     def parse(self, p):
@@ -139,7 +128,7 @@ class ClientHello(HandshakeMsg):
             cipherSpecsLength = p.get(2)
             sessionIDLength = p.get(2)
             randomLength = p.get(2)
-            self.cipher_suites = p.getFixList(3, int(cipherSpecsLength/3))
+            self.cipher_suites = p.getFixList(3, cipherSpecsLength//3)
             self.session_id = p.getFixBytes(sessionIDLength)
             self.random = p.getFixBytes(randomLength)
             if len(self.random) < 32:
@@ -161,18 +150,24 @@ class ClientHello(HandshakeMsg):
                 while soFar != totalExtLength:
                     extType = p.get(2)
                     extLength = p.get(2)
-                    if extType == 6:
+                    index1 = p.index
+                    if extType == ExtensionType.srp:
                         self.srp_username = bytesToString(p.getVarBytes(1))
-                    elif extType == 7:
+                    elif extType == ExtensionType.cert_type:
                         self.certificate_types = p.getVarList(1, 1)
+                    elif extType == ExtensionType.tack:
+                        self.tack = True
                     else:
                         p.getFixBytes(extLength)
+                    index2 = p.index
+                    if index2 - index1 != extLength:
+                        raise SyntaxError("Bad length for extension_data")
                     soFar += 4 + extLength
             p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.client_hello, trial)
+    def write(self):
+        w = Writer()
         w.add(self.client_version[0], 1)
         w.add(self.client_version[1], 1)
         w.addFixSeq(self.random, 1)
@@ -180,46 +175,44 @@ class ClientHello(HandshakeMsg):
         w.addVarSeq(self.cipher_suites, 2, 2)
         w.addVarSeq(self.compression_methods, 1, 1)
 
-        extLength = 0
+        w2 = Writer() # For Extensions
         if self.certificate_types and self.certificate_types != \
                 [CertificateType.x509]:
-            extLength += 5 + len(self.certificate_types)
+            w2.add(ExtensionType.cert_type, 2)
+            w2.add(len(self.certificate_types)+1, 2)
+            w2.addVarSeq(self.certificate_types, 1, 1)
         if self.srp_username:
-            extLength += 5 + len(self.srp_username)
-        if extLength > 0:
-            w.add(extLength, 2)
-
-        if self.certificate_types and self.certificate_types != \
-                [CertificateType.x509]:
-            w.add(7, 2)
-            w.add(len(self.certificate_types)+1, 2)
-            w.addVarSeq(self.certificate_types, 1, 1)
-        if self.srp_username:
-            w.add(6, 2)
-            w.add(len(self.srp_username)+1, 2)
-            w.addVarSeq(stringToBytes(self.srp_username), 1, 1)
-
-        return HandshakeMsg.postWrite(self, w, trial)
-
+            w2.add(ExtensionType.srp, 2)
+            w2.add(len(self.srp_username)+1, 2)
+            w2.addVarSeq(stringToBytes(self.srp_username), 1, 1)
+        if self.tack:
+            w2.add(ExtensionType.tack, 2)
+            w2.add(0, 2)
+        if len(w2.bytes):
+            w.add(len(w2.bytes), 2)
+            w.bytes += w2.bytes
+        return self.postWrite(w)
 
 class ServerHello(HandshakeMsg):
     def __init__(self):
-        self.contentType = ContentType.handshake
+        HandshakeMsg.__init__(self, HandshakeType.server_hello)
         self.server_version = (0,0)
         self.random = createByteArrayZeros(32)
         self.session_id = createByteArraySequence([])
         self.cipher_suite = 0
         self.certificate_type = CertificateType.x509
         self.compression_method = 0
+        self.tackExt = None 
 
     def create(self, version, random, session_id, cipher_suite,
-               certificate_type):
+               certificate_type, tackExt):
         self.server_version = version
         self.random = random
         self.session_id = session_id
         self.cipher_suite = cipher_suite
         self.certificate_type = certificate_type
         self.compression_method = 0
+        self.tackExt = tackExt
         return self
 
     def parse(self, p):
@@ -235,16 +228,21 @@ class ServerHello(HandshakeMsg):
             while soFar != totalExtLength:
                 extType = p.get(2)
                 extLength = p.get(2)
-                if extType == 7:
+                if extType == ExtensionType.cert_type:
+                    if extLength != 1:
+                        raise SyntaxError()
                     self.certificate_type = p.get(1)
+                elif extType == ExtensionType.tack and tackpyLoaded:
+                    self.tackExt = TACK_Extension()
+                    self.tackExt.parse(p.getFixBytes(extLength))
                 else:
                     p.getFixBytes(extLength)
                 soFar += 4 + extLength
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.server_hello, trial)
+    def write(self):
+        w = Writer()
         w.add(self.server_version[0], 1)
         w.add(self.server_version[1], 1)
         w.addFixSeq(self.random, 1)
@@ -252,26 +250,27 @@ class ServerHello(HandshakeMsg):
         w.add(self.cipher_suite, 2)
         w.add(self.compression_method, 1)
 
-        extLength = 0
+        w2 = Writer() # For Extensions
         if self.certificate_type and self.certificate_type != \
                 CertificateType.x509:
-            extLength += 5
+            w2.add(ExtensionType.cert_type, 2)
+            w2.add(1, 2)
+            w2.add(self.certificate_type, 1)
+        if self.tackExt:
+            b = self.tackExt.write()
+            w2.add(ExtensionType.tack, 2)
+            w2.add(len(b), 2)
+            w2.bytes += b
+        if len(w2.bytes):
+            w.add(len(w2.bytes), 2)
+            w.bytes += w2.bytes        
+        return self.postWrite(w)
 
-        if extLength != 0:
-            w.add(extLength, 2)
-
-        if self.certificate_type and self.certificate_type != \
-                CertificateType.x509:
-            w.add(7, 2)
-            w.add(1, 2)
-            w.add(self.certificate_type, 1)
-
-        return HandshakeMsg.postWrite(self, w, trial)
 
 class Certificate(HandshakeMsg):
     def __init__(self, certificateType):
+        HandshakeMsg.__init__(self, HandshakeType.certificate)
         self.certificateType = certificateType
-        self.contentType = ContentType.handshake
         self.certChain = None
 
     def create(self, certChain):
@@ -292,23 +291,14 @@ class Certificate(HandshakeMsg):
                 index += len(certBytes)+3
             if certificate_list:
                 self.certChain = X509CertChain(certificate_list)
-        elif self.certificateType == CertificateType.cryptoID:
-            s = bytesToString(p.getVarBytes(2))
-            if s:
-                try:
-                    import cryptoIDlib
-                except ImportError:
-                    raise SyntaxError(\
-                    "cryptoID cert chain received, cryptoIDlib not present")
-                self.certChain = cryptoIDlib.CertChain.CertChain().parse(s)
         else:
             raise AssertionError()
 
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.certificate, trial)
+    def write(self):
+        w = Writer()
         if self.certificateType == CertificateType.x509:
             chainLength = 0
             if self.certChain:
@@ -324,22 +314,17 @@ class Certificate(HandshakeMsg):
             for cert in certificate_list:
                 bytes = cert.writeBytes()
                 w.addVarSeq(bytes, 1, 3)
-        elif self.certificateType == CertificateType.cryptoID:
-            if self.certChain:
-                bytes = stringToBytes(self.certChain.write())
-            else:
-                bytes = createByteArraySequence([])
-            w.addVarSeq(bytes, 1, 2)
         else:
             raise AssertionError()
-        return HandshakeMsg.postWrite(self, w, trial)
+        return self.postWrite(w)
 
 class CertificateRequest(HandshakeMsg):
     def __init__(self):
-        self.contentType = ContentType.handshake
-        self.certificate_types = []
-        #treat as opaque bytes for now
-        self.certificate_authorities = createByteArraySequence([])
+        HandshakeMsg.__init__(self, HandshakeType.certificate_request)
+        #Apple's Secure Transport library rejects empty certificate_types, so
+        #default to rsa_sign.
+        self.certificate_types = [ClientCertificateType.rsa_sign]
+        self.certificate_authorities = []
 
     def create(self, certificate_types, certificate_authorities):
         self.certificate_types = certificate_types
@@ -349,21 +334,33 @@ class CertificateRequest(HandshakeMsg):
     def parse(self, p):
         p.startLengthCheck(3)
         self.certificate_types = p.getVarList(1, 1)
-        self.certificate_authorities = p.getVarBytes(2)
+        ca_list_length = p.get(2)
+        index = 0
+        self.certificate_authorities = []
+        while index != ca_list_length:
+          ca_bytes = p.getVarBytes(2)
+          self.certificate_authorities.append(ca_bytes)
+          index += len(ca_bytes)+2
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.certificate_request,
-                                  trial)
+    def write(self):
+        w = Writer()
         w.addVarSeq(self.certificate_types, 1, 1)
-        w.addVarSeq(self.certificate_authorities, 1, 2)
-        return HandshakeMsg.postWrite(self, w, trial)
+        caLength = 0
+        #determine length
+        for ca_dn in self.certificate_authorities:
+            caLength += len(ca_dn)+2
+        w.add(caLength, 2)
+        #add bytes
+        for ca_dn in self.certificate_authorities:
+            w.addVarSeq(ca_dn, 1, 2)
+        return self.postWrite(w)
 
 class ServerKeyExchange(HandshakeMsg):
     def __init__(self, cipherSuite):
+        HandshakeMsg.__init__(self, HandshakeType.server_key_exchange)
         self.cipherSuite = cipherSuite
-        self.contentType = ContentType.handshake
         self.srp_N = 0L
         self.srp_g = 0L
         self.srp_s = createByteArraySequence([])
@@ -383,21 +380,20 @@ class ServerKeyExchange(HandshakeMsg):
         self.srp_g = bytesToNumber(p.getVarBytes(2))
         self.srp_s = p.getVarBytes(1)
         self.srp_B = bytesToNumber(p.getVarBytes(2))
-        if self.cipherSuite in CipherSuite.srpRsaSuites:
+        if self.cipherSuite in CipherSuite.srpCertSuites:
             self.signature = p.getVarBytes(2)
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.server_key_exchange,
-                                  trial)
+    def write(self):
+        w = Writer()
         w.addVarSeq(numberToBytes(self.srp_N), 1, 2)
         w.addVarSeq(numberToBytes(self.srp_g), 1, 2)
         w.addVarSeq(self.srp_s, 1, 1)
         w.addVarSeq(numberToBytes(self.srp_B), 1, 2)
-        if self.cipherSuite in CipherSuite.srpRsaSuites:
+        if self.cipherSuite in CipherSuite.srpCertSuites:
             w.addVarSeq(self.signature, 1, 2)
-        return HandshakeMsg.postWrite(self, w, trial)
+        return self.postWrite(w)
 
     def hash(self, clientRandom, serverRandom):
         oldCipherSuite = self.cipherSuite
@@ -405,13 +401,13 @@ class ServerKeyExchange(HandshakeMsg):
         try:
             bytes = clientRandom + serverRandom + self.write()[4:]
             s = bytesToString(bytes)
-            return stringToBytes(md5.md5(s).digest() + sha.sha(s).digest())
+            return stringToBytes(md5(s).digest() + sha1(s).digest())
         finally:
             self.cipherSuite = oldCipherSuite
 
 class ServerHelloDone(HandshakeMsg):
     def __init__(self):
-        self.contentType = ContentType.handshake
+        HandshakeMsg.__init__(self, HandshakeType.server_hello_done)
 
     def create(self):
         return self
@@ -421,15 +417,15 @@ class ServerHelloDone(HandshakeMsg):
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.server_hello_done, trial)
-        return HandshakeMsg.postWrite(self, w, trial)
+    def write(self):
+        w = Writer()
+        return self.postWrite(w)
 
 class ClientKeyExchange(HandshakeMsg):
     def __init__(self, cipherSuite, version=None):
+        HandshakeMsg.__init__(self, HandshakeType.client_key_exchange)
         self.cipherSuite = cipherSuite
         self.version = version
-        self.contentType = ContentType.handshake
         self.srp_A = 0
         self.encryptedPreMasterSecret = createByteArraySequence([])
 
@@ -443,11 +439,10 @@ class ClientKeyExchange(HandshakeMsg):
 
     def parse(self, p):
         p.startLengthCheck(3)
-        if self.cipherSuite in CipherSuite.srpSuites + \
-                               CipherSuite.srpRsaSuites:
+        if self.cipherSuite in CipherSuite.srpAllSuites:
             self.srp_A = bytesToNumber(p.getVarBytes(2))
-        elif self.cipherSuite in CipherSuite.rsaSuites:
-            if self.version == (3,1) or self.version == (3,2):
+        elif self.cipherSuite in CipherSuite.certSuites:
+            if self.version in ((3,1), (3,2)):
                 self.encryptedPreMasterSecret = p.getVarBytes(2)
             elif self.version == (3,0):
                 self.encryptedPreMasterSecret = \
@@ -459,14 +454,12 @@ class ClientKeyExchange(HandshakeMsg):
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.client_key_exchange,
-                                  trial)
-        if self.cipherSuite in CipherSuite.srpSuites + \
-                               CipherSuite.srpRsaSuites:
+    def write(self):
+        w = Writer()
+        if self.cipherSuite in CipherSuite.srpAllSuites:
             w.addVarSeq(numberToBytes(self.srp_A), 1, 2)
-        elif self.cipherSuite in CipherSuite.rsaSuites:
-            if self.version == (3,1) or self.version == (3,2):
+        elif self.cipherSuite in CipherSuite.certSuites:
+            if self.version in ((3,1), (3,2)):
                 w.addVarSeq(self.encryptedPreMasterSecret, 1, 2)
             elif self.version == (3,0):
                 w.addFixSeq(self.encryptedPreMasterSecret, 1)
@@ -474,11 +467,11 @@ class ClientKeyExchange(HandshakeMsg):
                 raise AssertionError()
         else:
             raise AssertionError()
-        return HandshakeMsg.postWrite(self, w, trial)
+        return self.postWrite(w)
 
 class CertificateVerify(HandshakeMsg):
     def __init__(self):
-        self.contentType = ContentType.handshake
+        HandshakeMsg.__init__(self, HandshakeType.certificate_verify)
         self.signature = createByteArraySequence([])
 
     def create(self, signature):
@@ -491,13 +484,12 @@ class CertificateVerify(HandshakeMsg):
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.certificate_verify,
-                                  trial)
+    def write(self):
+        w = Writer()
         w.addVarSeq(self.signature, 1, 2)
-        return HandshakeMsg.postWrite(self, w, trial)
+        return self.postWrite(w)
 
-class ChangeCipherSpec(Msg):
+class ChangeCipherSpec:
     def __init__(self):
         self.contentType = ContentType.change_cipher_spec
         self.type = 1
@@ -512,15 +504,15 @@ class ChangeCipherSpec(Msg):
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = Msg.preWrite(self, trial)
+    def write(self):
+        w = Writer()
         w.add(self.type,1)
-        return Msg.postWrite(self, w, trial)
+        return w.bytes
 
 
 class Finished(HandshakeMsg):
     def __init__(self, version):
-        self.contentType = ContentType.handshake
+        HandshakeMsg.__init__(self, HandshakeType.finished)
         self.version = version
         self.verify_data = createByteArraySequence([])
 
@@ -539,12 +531,12 @@ class Finished(HandshakeMsg):
         p.stopLengthCheck()
         return self
 
-    def write(self, trial=False):
-        w = HandshakeMsg.preWrite(self, HandshakeType.finished, trial)
+    def write(self):
+        w = Writer()
         w.addFixSeq(self.verify_data, 1)
-        return HandshakeMsg.postWrite(self, w, trial)
+        return self.postWrite(w)
 
-class ApplicationData(Msg):
+class ApplicationData:
     def __init__(self):
         self.contentType = ContentType.application_data
         self.bytes = createByteArraySequence([])
@@ -552,6 +544,11 @@ class ApplicationData(Msg):
     def create(self, bytes):
         self.bytes = bytes
         return self
+        
+    def splitFirstByte(self):
+        newMsg = ApplicationData().create(self.bytes[:1])
+        self.bytes = self.bytes[1:]
+        return newMsg
 
     def parse(self, p):
         self.bytes = p.bytes
